/**
 * Copyright (c) 2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.su;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import java.util.List;
import java.util.Properties;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

import javax.xml.namespace.QName;

import org.junit.Test;
import org.ow2.petals.component.framework.AbstractComponent;
import org.ow2.petals.component.framework.jbidescriptor.generated.Consumes;
import org.ow2.petals.component.framework.jbidescriptor.generated.Jbi;
import org.ow2.petals.component.framework.jbidescriptor.generated.Services;

import com.ebmwebsourcing.easycommons.lang.reflect.ReflectionHelper;

/**
 * Unit test of {@link AbstractServiceUnitManager}
 * 
 * @author Christophe DENEUX - Linagora
 *
 */
public class AbstractServiceUnitManagerTest {

    private static final String NS = "http://petals.ow2.org/cdk/su/AbstractServiceUnitManagerTest";

    private static final String SU_NAME = "su-name";

    private static final String ITF_FOR_CONSUME_ONLY_BY_ITF_NAME = "itf-for-consume-only-by-itf";

    private static final QName ITF_FOR_CONSUME_ONLY_BY_ITF = new QName(NS, ITF_FOR_CONSUME_ONLY_BY_ITF_NAME);

    private static final String ITF_FOR_CONSUME_BY_SVC_NAME = "itf-for-consume-by-svc";

    private static final String SVC_FOR_CONSUME_BY_SVC_NAME = "svc-for-consume-by-svc";

    private static final QName ITF_FOR_CONSUME_BY_SVC = new QName(NS, ITF_FOR_CONSUME_BY_SVC_NAME);

    private static final QName SVC_FOR_CONSUME_BY_SVC = new QName(NS, SVC_FOR_CONSUME_BY_SVC_NAME);

    private static final String ITF_FOR_CONSUME_BY_EDP_NAME = "itf-for-consume-by-edp";

    private static final String SVC_FOR_CONSUME_BY_EDP_NAME = "svc-for-consume-by-edp";

    private static final String EDP_FOR_CONSUME_BY_EDP_NAME = "edp-for-consume-by-edp";

    private static final QName ITF_FOR_CONSUME_BY_EDP = new QName(NS, ITF_FOR_CONSUME_BY_EDP_NAME);

    private static final QName SVC_FOR_CONSUME_BY_EDP = new QName(NS, SVC_FOR_CONSUME_BY_EDP_NAME);

    private static final String ITF_FOR_CONSUME_WITHOUT_OP_NAME = "itf-for-consume-without-op";

    private static final String SVC_FOR_CONSUME_WITHOUT_OP_NAME = "svc-for-consume-without-op";

    private static final QName ITF_FOR_CONSUME_WITHOUT_OP = new QName(NS, ITF_FOR_CONSUME_WITHOUT_OP_NAME);

    private static final QName SVC_FOR_CONSUME_WITHOUT_OP = new QName(NS, SVC_FOR_CONSUME_WITHOUT_OP_NAME);

    private static final String OPERATION_1_NAME = "op-1";

    private static final QName OPERATION_1 = new QName(NS, OPERATION_1_NAME);

    private static final String OPERATION_2_NAME = "op-2";

    private static final QName OPERATION_2 = new QName(NS, OPERATION_2_NAME);

    public class ServiceUnitManager extends AbstractServiceUnitManager {

        public ServiceUnitManager() {
            super(new AbstractComponent() {

                @Override
                protected AbstractServiceUnitManager createServiceUnitManager() {
                    return null;
                }
            });
        }
    }

    /**
     * Check processing of {@link AbstractServiceUnitManager#getConsumesFromDestination(String, QName, QName)} (ie: no
     * operation is provided as parameter to
     * {@link AbstractServiceUnitManager#getConsumesFromDestination(String, QName, QName, QName)}).
     */
    @Test
    public void getConsumesFromDestionationWithoutOperation() {

        final AbstractServiceUnitManager asum = new ServiceUnitManager();
        this.initialize(asum);

        {
            final Consumes consumes = asum.getConsumesFromDestination(null, null, null);
            assertNull(consumes);
        }

        {
            final Consumes consumes = asum.getConsumesFromDestination(null, null, ITF_FOR_CONSUME_ONLY_BY_ITF);
            assertNotNull(consumes);
            assertEquals(ITF_FOR_CONSUME_ONLY_BY_ITF, consumes.getInterfaceName());
            assertNull(consumes.getServiceName());
            assertNull(consumes.getEndpointName());
            assertNull(consumes.getOperation());
        }

        {
            final Consumes consumes = asum.getConsumesFromDestination(null, SVC_FOR_CONSUME_BY_SVC,
                    ITF_FOR_CONSUME_BY_SVC);
            assertNotNull(consumes);
            assertEquals(ITF_FOR_CONSUME_BY_SVC, consumes.getInterfaceName());
            assertEquals(SVC_FOR_CONSUME_BY_SVC, consumes.getServiceName());
            assertNull(consumes.getEndpointName());
            assertNull(consumes.getOperation());
        }

        {
            final Consumes consumes = asum.getConsumesFromDestination(null, SVC_FOR_CONSUME_BY_SVC, null);
            assertNotNull(consumes);
            assertEquals(ITF_FOR_CONSUME_BY_SVC, consumes.getInterfaceName());
            assertEquals(SVC_FOR_CONSUME_BY_SVC, consumes.getServiceName());
            assertNull(consumes.getEndpointName());
            assertNull(consumes.getOperation());
        }

        {
            final Consumes consumes = asum.getConsumesFromDestination(EDP_FOR_CONSUME_BY_EDP_NAME, null, null);
            assertNotNull(consumes);
            assertEquals(ITF_FOR_CONSUME_BY_EDP, consumes.getInterfaceName());
            assertEquals(SVC_FOR_CONSUME_BY_EDP, consumes.getServiceName());
            assertEquals(EDP_FOR_CONSUME_BY_EDP_NAME, consumes.getEndpointName());
            assertNull(consumes.getOperation());
        }

        {
            final Consumes consumes = asum.getConsumesFromDestination(EDP_FOR_CONSUME_BY_EDP_NAME,
                    SVC_FOR_CONSUME_BY_EDP, null);
            assertNotNull(consumes);
            assertEquals(ITF_FOR_CONSUME_BY_EDP, consumes.getInterfaceName());
            assertEquals(SVC_FOR_CONSUME_BY_EDP, consumes.getServiceName());
            assertEquals(EDP_FOR_CONSUME_BY_EDP_NAME, consumes.getEndpointName());
            assertNull(consumes.getOperation());
        }

        {
            final Consumes consumes = asum.getConsumesFromDestination(EDP_FOR_CONSUME_BY_EDP_NAME,
                    SVC_FOR_CONSUME_BY_EDP, ITF_FOR_CONSUME_BY_EDP);
            assertNotNull(consumes);
            assertEquals(ITF_FOR_CONSUME_BY_EDP, consumes.getInterfaceName());
            assertEquals(SVC_FOR_CONSUME_BY_EDP, consumes.getServiceName());
            assertEquals(EDP_FOR_CONSUME_BY_EDP_NAME, consumes.getEndpointName());
            assertNull(consumes.getOperation());
        }

        {
            final Consumes consumes = asum.getConsumesFromDestination(EDP_FOR_CONSUME_BY_EDP_NAME, null,
                    ITF_FOR_CONSUME_BY_EDP);
            assertNotNull(consumes);
            assertEquals(ITF_FOR_CONSUME_BY_EDP, consumes.getInterfaceName());
            assertEquals(SVC_FOR_CONSUME_BY_EDP, consumes.getServiceName());
            assertEquals(EDP_FOR_CONSUME_BY_EDP_NAME, consumes.getEndpointName());
            assertNull(consumes.getOperation());
        }
    }

    /**
     * Check processing of {@link AbstractServiceUnitManager#getConsumesFromDestination(String, QName, QName, QName)}
     * when an operation is provided as parameter.
     */
    @Test
    public void getConsumesFromDestionationWithOperation() {

        final AbstractServiceUnitManager asum = new ServiceUnitManager();
        this.initialize(asum);

        {
            final Consumes consumes = asum.getConsumesFromDestination(null, null, null, null);
            assertNull(consumes);
        }

        {
            final Consumes consumes = asum.getConsumesFromDestination(null, null, ITF_FOR_CONSUME_ONLY_BY_ITF, null);
            assertNotNull(consumes);
            assertEquals(ITF_FOR_CONSUME_ONLY_BY_ITF, consumes.getInterfaceName());
            assertNull(consumes.getServiceName());
            assertNull(consumes.getEndpointName());
            assertNull(consumes.getOperation());
        }
        {
            final Consumes consumes = asum.getConsumesFromDestination(null, null, ITF_FOR_CONSUME_ONLY_BY_ITF,
                    OPERATION_1);
            assertNotNull(consumes);
            assertEquals(ITF_FOR_CONSUME_ONLY_BY_ITF, consumes.getInterfaceName());
            assertNull(consumes.getServiceName());
            assertNull(consumes.getEndpointName());
            assertEquals(OPERATION_1, consumes.getOperation());
        }
        {
            final Consumes consumes = asum.getConsumesFromDestination(null, null, ITF_FOR_CONSUME_ONLY_BY_ITF,
                    OPERATION_2);
            assertNotNull(consumes);
            assertEquals(ITF_FOR_CONSUME_ONLY_BY_ITF, consumes.getInterfaceName());
            assertNull(consumes.getServiceName());
            assertNull(consumes.getEndpointName());
            assertEquals(OPERATION_2, consumes.getOperation());
        }

        {
            final Consumes consumes = asum.getConsumesFromDestination(null, SVC_FOR_CONSUME_BY_SVC,
                    ITF_FOR_CONSUME_BY_SVC, null);
            assertNotNull(consumes);
            assertEquals(ITF_FOR_CONSUME_BY_SVC, consumes.getInterfaceName());
            assertEquals(SVC_FOR_CONSUME_BY_SVC, consumes.getServiceName());
            assertNull(consumes.getEndpointName());
            assertNull(consumes.getOperation());
        }
        {
            final Consumes consumes = asum.getConsumesFromDestination(null, SVC_FOR_CONSUME_BY_SVC,
                    ITF_FOR_CONSUME_BY_SVC, OPERATION_1);
            assertNotNull(consumes);
            assertEquals(ITF_FOR_CONSUME_BY_SVC, consumes.getInterfaceName());
            assertEquals(SVC_FOR_CONSUME_BY_SVC, consumes.getServiceName());
            assertNull(consumes.getEndpointName());
            assertEquals(OPERATION_1, consumes.getOperation());
        }
        {
            final Consumes consumes = asum.getConsumesFromDestination(null, SVC_FOR_CONSUME_BY_SVC,
                    ITF_FOR_CONSUME_BY_SVC, OPERATION_2);
            assertNotNull(consumes);
            assertEquals(ITF_FOR_CONSUME_BY_SVC, consumes.getInterfaceName());
            assertEquals(SVC_FOR_CONSUME_BY_SVC, consumes.getServiceName());
            assertNull(consumes.getEndpointName());
            assertEquals(OPERATION_2, consumes.getOperation());
        }

        {
            final Consumes consumes = asum.getConsumesFromDestination(null, SVC_FOR_CONSUME_BY_SVC, null, null);
            assertNotNull(consumes);
            assertEquals(ITF_FOR_CONSUME_BY_SVC, consumes.getInterfaceName());
            assertEquals(SVC_FOR_CONSUME_BY_SVC, consumes.getServiceName());
            assertNull(consumes.getEndpointName());
            assertNull(consumes.getOperation());
        }

        {
            final Consumes consumes = asum.getConsumesFromDestination(EDP_FOR_CONSUME_BY_EDP_NAME,
                    SVC_FOR_CONSUME_BY_SVC, null, null);
            assertNotNull(consumes);
            assertEquals(ITF_FOR_CONSUME_BY_EDP, consumes.getInterfaceName());
            assertEquals(SVC_FOR_CONSUME_BY_EDP, consumes.getServiceName());
            assertEquals(EDP_FOR_CONSUME_BY_EDP_NAME, consumes.getEndpointName());
            assertNull(consumes.getOperation());
        }

        {
            final Consumes consumes = asum.getConsumesFromDestination(EDP_FOR_CONSUME_BY_EDP_NAME,
                    SVC_FOR_CONSUME_BY_SVC, null, OPERATION_1);
            assertNotNull(consumes);
            assertEquals(ITF_FOR_CONSUME_BY_EDP, consumes.getInterfaceName());
            assertEquals(SVC_FOR_CONSUME_BY_EDP, consumes.getServiceName());
            assertEquals(EDP_FOR_CONSUME_BY_EDP_NAME, consumes.getEndpointName());
            assertEquals(OPERATION_1, consumes.getOperation());
        }

        {
            final Consumes consumes = asum.getConsumesFromDestination(EDP_FOR_CONSUME_BY_EDP_NAME,
                    SVC_FOR_CONSUME_BY_SVC, null, OPERATION_2);
            assertNotNull(consumes);
            assertEquals(ITF_FOR_CONSUME_BY_EDP, consumes.getInterfaceName());
            assertEquals(SVC_FOR_CONSUME_BY_EDP, consumes.getServiceName());
            assertEquals(EDP_FOR_CONSUME_BY_EDP_NAME, consumes.getEndpointName());
            assertEquals(OPERATION_2, consumes.getOperation());
        }

        {
            final Consumes consumes = asum.getConsumesFromDestination(EDP_FOR_CONSUME_BY_EDP_NAME,
                    SVC_FOR_CONSUME_BY_SVC, ITF_FOR_CONSUME_ONLY_BY_ITF, null);
            assertNotNull(consumes);
            assertEquals(ITF_FOR_CONSUME_BY_EDP, consumes.getInterfaceName());
            assertEquals(SVC_FOR_CONSUME_BY_EDP, consumes.getServiceName());
            assertEquals(EDP_FOR_CONSUME_BY_EDP_NAME, consumes.getEndpointName());
            assertNull(consumes.getOperation());
        }

        {
            final Consumes consumes = asum.getConsumesFromDestination(EDP_FOR_CONSUME_BY_EDP_NAME,
                    SVC_FOR_CONSUME_BY_SVC, ITF_FOR_CONSUME_ONLY_BY_ITF, OPERATION_1);
            assertNotNull(consumes);
            assertEquals(ITF_FOR_CONSUME_BY_EDP, consumes.getInterfaceName());
            assertEquals(SVC_FOR_CONSUME_BY_EDP, consumes.getServiceName());
            assertEquals(EDP_FOR_CONSUME_BY_EDP_NAME, consumes.getEndpointName());
            assertEquals(OPERATION_1, consumes.getOperation());
        }

        {
            final Consumes consumes = asum.getConsumesFromDestination(EDP_FOR_CONSUME_BY_EDP_NAME,
                    SVC_FOR_CONSUME_BY_SVC, ITF_FOR_CONSUME_ONLY_BY_ITF, OPERATION_2);
            assertNotNull(consumes);
            assertEquals(ITF_FOR_CONSUME_BY_EDP, consumes.getInterfaceName());
            assertEquals(SVC_FOR_CONSUME_BY_EDP, consumes.getServiceName());
            assertEquals(EDP_FOR_CONSUME_BY_EDP_NAME, consumes.getEndpointName());
            assertEquals(OPERATION_2, consumes.getOperation());
        }

        {
            final Consumes consumes = asum.getConsumesFromDestination(EDP_FOR_CONSUME_BY_EDP_NAME, null,
                    ITF_FOR_CONSUME_ONLY_BY_ITF, null);
            assertNotNull(consumes);
            assertEquals(ITF_FOR_CONSUME_BY_EDP, consumes.getInterfaceName());
            assertEquals(SVC_FOR_CONSUME_BY_EDP, consumes.getServiceName());
            assertEquals(EDP_FOR_CONSUME_BY_EDP_NAME, consumes.getEndpointName());
            assertNull(consumes.getOperation());
        }

        {
            final Consumes consumes = asum.getConsumesFromDestination(EDP_FOR_CONSUME_BY_EDP_NAME, null,
                    ITF_FOR_CONSUME_ONLY_BY_ITF, OPERATION_1);
            assertNotNull(consumes);
            assertEquals(ITF_FOR_CONSUME_BY_EDP, consumes.getInterfaceName());
            assertEquals(SVC_FOR_CONSUME_BY_EDP, consumes.getServiceName());
            assertEquals(EDP_FOR_CONSUME_BY_EDP_NAME, consumes.getEndpointName());
            assertEquals(OPERATION_1, consumes.getOperation());
        }

        {
            final Consumes consumes = asum.getConsumesFromDestination(EDP_FOR_CONSUME_BY_EDP_NAME, null,
                    ITF_FOR_CONSUME_ONLY_BY_ITF, OPERATION_2);
            assertNotNull(consumes);
            assertEquals(ITF_FOR_CONSUME_BY_EDP, consumes.getInterfaceName());
            assertEquals(SVC_FOR_CONSUME_BY_EDP, consumes.getServiceName());
            assertEquals(EDP_FOR_CONSUME_BY_EDP_NAME, consumes.getEndpointName());
            assertEquals(OPERATION_2, consumes.getOperation());
        }
    }

    /**
     * Check processing of
     * {@link AbstractServiceUnitManager#getConsumesFromExchange(org.ow2.petals.component.framework.api.message.Exchange)}
     * when no operation is defined into the provided exchange.
     */
    @Test
    public void getConsumesFromExchangeWithoutOperation() {

        final AbstractServiceUnitManager asum = new ServiceUnitManager();
        this.initialize(asum);

        {
            final Consumes consumes = asum.getConsumesFromExchange(new MockExchange(ITF_FOR_CONSUME_ONLY_BY_ITF, null, null, null));
            assertNotNull(consumes);
            assertEquals(ITF_FOR_CONSUME_ONLY_BY_ITF, consumes.getInterfaceName());
            assertNull(consumes.getServiceName());
            assertNull(consumes.getEndpointName());
            assertNull(consumes.getOperation());
        }

        {
            final Consumes consumes = asum.getConsumesFromExchange(
                    new MockExchange(ITF_FOR_CONSUME_BY_SVC, SVC_FOR_CONSUME_BY_SVC, null, null));
            assertNotNull(consumes);
            assertEquals(ITF_FOR_CONSUME_BY_SVC, consumes.getInterfaceName());
            assertEquals(SVC_FOR_CONSUME_BY_SVC, consumes.getServiceName());
            assertNull(consumes.getEndpointName());
            assertNull(consumes.getOperation());
        }

        {
            final Consumes consumes = asum.getConsumesFromExchange(new MockExchange(null, SVC_FOR_CONSUME_BY_SVC, null, null));
            assertNotNull(consumes);
            assertEquals(ITF_FOR_CONSUME_BY_SVC, consumes.getInterfaceName());
            assertEquals(SVC_FOR_CONSUME_BY_SVC, consumes.getServiceName());
            assertNull(consumes.getEndpointName());
            assertNull(consumes.getOperation());
        }

        {
            final Consumes consumes = asum.getConsumesFromExchange(new MockExchange(null, null, EDP_FOR_CONSUME_BY_EDP_NAME, null));
            assertNotNull(consumes);
            assertEquals(ITF_FOR_CONSUME_BY_EDP, consumes.getInterfaceName());
            assertEquals(SVC_FOR_CONSUME_BY_EDP, consumes.getServiceName());
            assertEquals(EDP_FOR_CONSUME_BY_EDP_NAME, consumes.getEndpointName());
            assertNull(consumes.getOperation());
        }

        {
            final Consumes consumes = asum.getConsumesFromExchange(new MockExchange(null, SVC_FOR_CONSUME_BY_EDP, EDP_FOR_CONSUME_BY_EDP_NAME, null));
            assertNotNull(consumes);
            assertEquals(ITF_FOR_CONSUME_BY_EDP, consumes.getInterfaceName());
            assertEquals(SVC_FOR_CONSUME_BY_EDP, consumes.getServiceName());
            assertEquals(EDP_FOR_CONSUME_BY_EDP_NAME, consumes.getEndpointName());
            assertNull(consumes.getOperation());
        }

        {
            final Consumes consumes = asum.getConsumesFromExchange(new MockExchange(ITF_FOR_CONSUME_BY_EDP, SVC_FOR_CONSUME_BY_EDP, EDP_FOR_CONSUME_BY_EDP_NAME, null));
            assertNotNull(consumes);
            assertEquals(ITF_FOR_CONSUME_BY_EDP, consumes.getInterfaceName());
            assertEquals(SVC_FOR_CONSUME_BY_EDP, consumes.getServiceName());
            assertEquals(EDP_FOR_CONSUME_BY_EDP_NAME, consumes.getEndpointName());
            assertNull(consumes.getOperation());
        }

        {
            final Consumes consumes = asum.getConsumesFromExchange(new MockExchange(ITF_FOR_CONSUME_BY_EDP, null, EDP_FOR_CONSUME_BY_EDP_NAME, null));
            assertNotNull(consumes);
            assertEquals(ITF_FOR_CONSUME_BY_EDP, consumes.getInterfaceName());
            assertEquals(SVC_FOR_CONSUME_BY_EDP, consumes.getServiceName());
            assertEquals(EDP_FOR_CONSUME_BY_EDP_NAME, consumes.getEndpointName());
            assertNull(consumes.getOperation());
        }
    }

    /**
     * Check processing of
     * {@link AbstractServiceUnitManager#getConsumesFromExchange(org.ow2.petals.component.framework.api.message.Exchange)}
     * when an operation is defined into the provided exchange.
     */
    @Test
    public void getConsumesFromExchangeWithOperation() {

        final AbstractServiceUnitManager asum = new ServiceUnitManager();
        this.initialize(asum);

        {
            final Consumes consumes = asum
                    .getConsumesFromExchange(new MockExchange(ITF_FOR_CONSUME_ONLY_BY_ITF, null, null, OPERATION_1));
            assertNotNull(consumes);
            assertEquals(ITF_FOR_CONSUME_ONLY_BY_ITF, consumes.getInterfaceName());
            assertNull(consumes.getServiceName());
            assertNull(consumes.getEndpointName());
            assertEquals(OPERATION_1, consumes.getOperation());
        }

        {
            final Consumes consumes = asum.getConsumesFromExchange(
                    new MockExchange(ITF_FOR_CONSUME_BY_SVC, SVC_FOR_CONSUME_BY_SVC, null, OPERATION_2));
            assertNotNull(consumes);
            assertEquals(ITF_FOR_CONSUME_BY_SVC, consumes.getInterfaceName());
            assertEquals(SVC_FOR_CONSUME_BY_SVC, consumes.getServiceName());
            assertNull(consumes.getEndpointName());
            assertEquals(OPERATION_2, consumes.getOperation());
        }

        {
            final Consumes consumes = asum
                    .getConsumesFromExchange(new MockExchange(null, SVC_FOR_CONSUME_BY_SVC, null, OPERATION_1));
            assertNotNull(consumes);
            assertEquals(ITF_FOR_CONSUME_BY_SVC, consumes.getInterfaceName());
            assertEquals(SVC_FOR_CONSUME_BY_SVC, consumes.getServiceName());
            assertNull(consumes.getEndpointName());
            assertEquals(OPERATION_1, consumes.getOperation());
        }

        {
            final Consumes consumes = asum
                    .getConsumesFromExchange(new MockExchange(null, null, EDP_FOR_CONSUME_BY_EDP_NAME, OPERATION_2));
            assertNotNull(consumes);
            assertEquals(ITF_FOR_CONSUME_BY_EDP, consumes.getInterfaceName());
            assertEquals(SVC_FOR_CONSUME_BY_EDP, consumes.getServiceName());
            assertEquals(EDP_FOR_CONSUME_BY_EDP_NAME, consumes.getEndpointName());
            assertEquals(OPERATION_2, consumes.getOperation());
        }

        {
            final Consumes consumes = asum.getConsumesFromExchange(
                    new MockExchange(null, SVC_FOR_CONSUME_BY_EDP, EDP_FOR_CONSUME_BY_EDP_NAME, OPERATION_1));
            assertNotNull(consumes);
            assertEquals(ITF_FOR_CONSUME_BY_EDP, consumes.getInterfaceName());
            assertEquals(SVC_FOR_CONSUME_BY_EDP, consumes.getServiceName());
            assertEquals(EDP_FOR_CONSUME_BY_EDP_NAME, consumes.getEndpointName());
            assertEquals(OPERATION_1, consumes.getOperation());
        }

        {
            final Consumes consumes = asum.getConsumesFromExchange(new MockExchange(ITF_FOR_CONSUME_BY_EDP,
                    SVC_FOR_CONSUME_BY_EDP, EDP_FOR_CONSUME_BY_EDP_NAME, OPERATION_2));
            assertNotNull(consumes);
            assertEquals(ITF_FOR_CONSUME_BY_EDP, consumes.getInterfaceName());
            assertEquals(SVC_FOR_CONSUME_BY_EDP, consumes.getServiceName());
            assertEquals(EDP_FOR_CONSUME_BY_EDP_NAME, consumes.getEndpointName());
            assertEquals(OPERATION_2, consumes.getOperation());
        }

        {
            final Consumes consumes = asum.getConsumesFromExchange(
                    new MockExchange(ITF_FOR_CONSUME_BY_EDP, null, EDP_FOR_CONSUME_BY_EDP_NAME, OPERATION_1));
            assertNotNull(consumes);
            assertEquals(ITF_FOR_CONSUME_BY_EDP, consumes.getInterfaceName());
            assertEquals(SVC_FOR_CONSUME_BY_EDP, consumes.getServiceName());
            assertEquals(EDP_FOR_CONSUME_BY_EDP_NAME, consumes.getEndpointName());
            assertEquals(OPERATION_1, consumes.getOperation());
        }
        {
            final Consumes consumes = asum.getConsumesFromExchange(
                    new MockExchange(ITF_FOR_CONSUME_WITHOUT_OP, SVC_FOR_CONSUME_WITHOUT_OP, null, null));
            assertNotNull(consumes);
            assertEquals(ITF_FOR_CONSUME_WITHOUT_OP, consumes.getInterfaceName());
            assertEquals(SVC_FOR_CONSUME_WITHOUT_OP, consumes.getServiceName());
            assertNull(consumes.getEndpointName());
            assertNull(consumes.getOperation());
        }
        {
            final Consumes consumes = asum.getConsumesFromExchange(new MockExchange(ITF_FOR_CONSUME_WITHOUT_OP,
                    SVC_FOR_CONSUME_WITHOUT_OP, null, new QName("urn:dummy", "an-operation")));
            assertNotNull(consumes);
            assertEquals(ITF_FOR_CONSUME_WITHOUT_OP, consumes.getInterfaceName());
            assertEquals(SVC_FOR_CONSUME_WITHOUT_OP, consumes.getServiceName());
            assertNull(consumes.getEndpointName());
            assertNull(consumes.getOperation());
        }
    }

    private void initialize(final AbstractServiceUnitManager asum) {

        final Object serviceUnitDataHandlersObj = ReflectionHelper.getFieldValue(AbstractServiceUnitManager.class, asum,
                "serviceUnitDataHandlers", false);
        assertNotNull(serviceUnitDataHandlersObj);
        assertTrue(serviceUnitDataHandlersObj instanceof ConcurrentHashMap);
        final ConcurrentMap<String, ServiceUnitDataHandler> serviceUnitDataHandlers = (ConcurrentMap) serviceUnitDataHandlersObj;

        final Jbi jbi = new Jbi();
        final Services services = new Services();
        jbi.setServices(services);
        final List<Consumes> consumes = services.getConsumes();

        final Consumes consumeItfOnly = new Consumes();
        consumeItfOnly.setInterfaceName(ITF_FOR_CONSUME_ONLY_BY_ITF);
        consumes.add(consumeItfOnly);

        final Consumes consumeItfOnlyOp1 = new Consumes();
        consumeItfOnlyOp1.setInterfaceName(ITF_FOR_CONSUME_ONLY_BY_ITF);
        consumeItfOnlyOp1.setOperation(OPERATION_1);
        consumes.add(consumeItfOnlyOp1);

        final Consumes consumeItfOnlyOp2 = new Consumes();
        consumeItfOnlyOp2.setInterfaceName(ITF_FOR_CONSUME_ONLY_BY_ITF);
        consumeItfOnlyOp2.setOperation(OPERATION_2);
        consumes.add(consumeItfOnlyOp2);

        final Consumes consumeSvc = new Consumes();
        consumeSvc.setInterfaceName(ITF_FOR_CONSUME_BY_SVC);
        consumeSvc.setServiceName(SVC_FOR_CONSUME_BY_SVC);
        consumes.add(consumeSvc);

        final Consumes consumeSvcOp1 = new Consumes();
        consumeSvcOp1.setInterfaceName(ITF_FOR_CONSUME_BY_SVC);
        consumeSvcOp1.setServiceName(SVC_FOR_CONSUME_BY_SVC);
        consumeSvcOp1.setOperation(OPERATION_1);
        consumes.add(consumeSvcOp1);

        final Consumes consumeSvcOp2 = new Consumes();
        consumeSvcOp2.setInterfaceName(ITF_FOR_CONSUME_BY_SVC);
        consumeSvcOp2.setServiceName(SVC_FOR_CONSUME_BY_SVC);
        consumeSvcOp2.setOperation(OPERATION_2);
        consumes.add(consumeSvcOp2);

        final Consumes consumeSvcEdp = new Consumes();
        consumeSvcEdp.setInterfaceName(ITF_FOR_CONSUME_BY_EDP);
        consumeSvcEdp.setServiceName(SVC_FOR_CONSUME_BY_EDP);
        consumeSvcEdp.setEndpointName(EDP_FOR_CONSUME_BY_EDP_NAME);
        consumes.add(consumeSvcEdp);

        final Consumes consumeSvcEdpOp1 = new Consumes();
        consumeSvcEdpOp1.setInterfaceName(ITF_FOR_CONSUME_BY_EDP);
        consumeSvcEdpOp1.setServiceName(SVC_FOR_CONSUME_BY_EDP);
        consumeSvcEdpOp1.setEndpointName(EDP_FOR_CONSUME_BY_EDP_NAME);
        consumeSvcEdpOp1.setOperation(OPERATION_1);
        consumes.add(consumeSvcEdpOp1);

        final Consumes consumeSvcEdpOp2 = new Consumes();
        consumeSvcEdpOp2.setInterfaceName(ITF_FOR_CONSUME_BY_EDP);
        consumeSvcEdpOp2.setServiceName(SVC_FOR_CONSUME_BY_EDP);
        consumeSvcEdpOp2.setEndpointName(EDP_FOR_CONSUME_BY_EDP_NAME);
        consumeSvcEdpOp2.setOperation(OPERATION_2);
        consumes.add(consumeSvcEdpOp2);

        final Consumes consumeWithoutOp = new Consumes();
        consumeWithoutOp.setInterfaceName(ITF_FOR_CONSUME_WITHOUT_OP);
        consumeWithoutOp.setServiceName(SVC_FOR_CONSUME_WITHOUT_OP);
        consumes.add(consumeWithoutOp);

        serviceUnitDataHandlers.put(SU_NAME, new ServiceUnitDataHandler(SU_NAME, null, jbi, new Properties()));

    }

}
